pub mod event;

use sycamore::prelude::*;
use web_sys::Event;

#[derive(Prop)]
pub struct PageProps<'a, G: Html> {
    pub children: Children<'a, G>,
}

#[component]
pub fn Page<'a, G: Html>(cx: Scope<'a>, props: PageProps<'a, G>) -> View<G> {
    let children = props.children.call(cx);

    view! { cx,
        div(class="max-w-7xl mx-auto") { (children) }
    }
}

#[derive(Prop)]
pub struct Props<F: FnMut(Event)> {
    pub onclick: F,
    #[builder(default)]
    pub text: String,
    #[builder(default)]
    pub icon: String,
}

#[component]
pub fn Button<'a, G: Html, F: 'a + FnMut(Event)>(cx: Scope<'a>, props: Props<F>) -> View<G> {
    let mut icon_class = String::from("mdi text-lg ");

    if !props.icon.is_empty() {
        icon_class.push_str(&props.icon);
    }

    view! { cx,
        button(class="bg-gray-700 hover:bg-gray-800 text-gray-400 font-bold py-2 px-2 rounded inline-flex items-center",on:click=props.onclick) {
            span(class=icon_class)
            span { (props.text) }
        }
    }
}

#[derive(Prop)]
pub struct TableProps<'a, G: Html> {
    pub headers: Vec<String>,

    pub children: Children<'a, G>,
}

#[component]
pub fn Table<'a, G: Html>(cx: Scope<'a>, props: TableProps<'a, G>) -> View<G> {
    let children = props.children.call(cx);

    view! { cx,
        div(class="inline-block min-w-full py-2 align-middle") {
            div(class="overflow-hidden shadow ring-1 ring-black ring-opacity-5 md:rounded-lg") {
                table(class="min-w-full divide-y divide-gray-600") {
                    thead(class="bg-gray-800") {
                        tr {
                            (View::new_fragment(props.headers.iter().cloned().map(|header| view! { cx,
                                th(
                                    scope="col",
                                    class="px-3 py-3.5 text-left
                                    text-sm font-semibold text-gray-400"
                                ) {
                                    (header)
                                }
                            }).collect()))
                        }
                    }
                    tbody(class="divide-y divide-gray-600 bg-gray-700") {
                        (children)
                    }
                }
            }
        }
    }
}

#[derive(Prop)]
pub struct BlockProps<'a, G: Html> {
    title: String,
    children: Children<'a, G>,
}

#[component]
pub fn Block<'a, G: Html>(cx: Scope<'a>, props: BlockProps<'a, G>) -> View<G> {
    let children = props.children.call(cx);
    let open = create_signal(cx, false);

    let class = create_memo(cx, || {
        if *open.get() {
            "overflow-hidden"
        } else {
            "overflow-hidden max-h-1"
        }
    });

    view! { cx,
        div(class="px-3 py-3 rounded-lg border-solid border-gray-800 border-2 my-3") {
            p(class="cursor-pointer text-lg",on:click=move |_| open.set(!*open.get())) { (props.title) }
            div(class=class) {
                br {}
                div {
                    (children)
                }
            }
        }
    }
}

/*
#[function_component(Loading)]
pub fn loading() -> Html {
    html! {
        <svg role="status" class="inline h-8 w-8 animate-spin mr-2 text-gray-200 dark:text-gray-600 fill-gray-400"
            viewBox="0 0 100 101" fill="none" xmlns="http://www.w3.org/2000/svg">
            <path
                d="M100 50.5908C100 78.2051 77.6142 100.591 50 100.591C22.3858 100.591 0 78.2051 0 50.5908C0 22.9766 22.3858 0.59082 50 0.59082C77.6142 0.59082 100 22.9766 100 50.5908ZM9.08144 50.5908C9.08144 73.1895 27.4013 91.5094 50 91.5094C72.5987 91.5094 90.9186 73.1895 90.9186 50.5908C90.9186 27.9921 72.5987 9.67226 50 9.67226C27.4013 9.67226 9.08144 27.9921 9.08144 50.5908Z"
                fill="currentColor" />
            <path
                d="M93.9676 39.0409C96.393 38.4038 97.8624 35.9116 97.0079 33.5539C95.2932 28.8227 92.871 24.3692 89.8167 20.348C85.8452 15.1192 80.8826 10.7238 75.2124 7.41289C69.5422 4.10194 63.2754 1.94025 56.7698 1.05124C51.7666 0.367541 46.6976 0.446843 41.7345 1.27873C39.2613 1.69328 37.813 4.19778 38.4501 6.62326C39.0873 9.04874 41.5694 10.4717 44.0505 10.1071C47.8511 9.54855 51.7191 9.52689 55.5402 10.0491C60.8642 10.7766 65.9928 12.5457 70.6331 15.2552C75.2735 17.9648 79.3347 21.5619 82.5849 25.841C84.9175 28.9121 86.7997 32.2913 88.1811 35.8758C89.083 38.2158 91.5421 39.6781 93.9676 39.0409Z"
                fill="currentFill" />
        </svg>
    }
}
*/
