use crate::components::{Button, Page, Table};
use lan_party_core::user::User;
use log::debug;
use reqwasm::http::Method;
use sycamore::{futures::spawn_local_scoped, prelude::*};
use web_sys::Event;

use crate::util::api_request;

#[component]
pub fn UsersPage<'a, G: Html>(cx: Scope<'a>) -> View<G> {
    let users = create_signal(cx, Vec::<User>::new());
    let headers = vec!["Username".into(), "Score".into(), "".into()];

    let score_edit = create_signal(cx, Option::<String>::None);
    let new_score = create_signal(cx, String::new());
    let new_username = create_signal(cx, String::new());

    spawn_local_scoped(cx, async move {
        users.set(
            api_request::<_, Vec<User>>(Method::GET, "/user", Option::<()>::None)
                .await
                .map(|inner| inner.unwrap())
                .unwrap(),
        );
    });

    let ondelete = move |name: String| {
        move |event: Event| {
            let name = name.clone();
            spawn_local_scoped(cx, async move {
                debug!("Delete {:#?}", event);
                let users_ref = users.get();
                let user: &User = users_ref.iter().find(|user| user.name == name).unwrap();
                api_request::<_, ()>(
                    Method::DELETE,
                    &format!("/user/{}", user.name),
                    Option::<()>::None,
                )
                .await
                .unwrap();
                let cloned = (*users_ref)
                    .clone()
                    .iter()
                    .cloned()
                    .filter(|u| u.name != user.name)
                    .collect();
                users.set(cloned);
            });
        }
    };

    let oncheck = move |_| {
        spawn_local_scoped(cx, async move {
            if let (Some(score_edit), Ok(score)) =
                (score_edit.get().as_ref(), new_score.get().parse())
            {
                let score: i64 = score;
                let users_ref = users.get();
                let user: &User = users_ref
                    .iter()
                    .find(|user| &user.name == score_edit)
                    .unwrap();
                api_request::<_, ()>(
                    Method::POST,
                    &format!("/user/{}/score", user.name),
                    Some(score),
                )
                .await
                .unwrap();
                let cloned = (*users_ref).clone();
                let new_users: Vec<_> = cloned
                    .into_iter()
                    .map(|mut user| {
                        if &user.name == score_edit {
                            user.score = score
                        }
                        user
                    })
                    .collect();
                users.set(new_users);
            }
            score_edit.set(None);
        })
    };

    let onadd = move |_| {
        spawn_local_scoped(cx, async move {
            let user = api_request::<String, User>(
                Method::POST,
                "/user",
                Some((*new_username).get().as_ref().clone()),
            )
            .await
            .unwrap();
            let mut cloned = (*users).get().as_ref().clone();
            cloned.push(user.unwrap());
            users.set(cloned);
        });
    };

    view! { cx,
        Page {
            Table(headers=headers) {
                Keyed(
                    iterable=users,
                    view=move |cx, user| {
                        let user = create_ref(cx, user);
                        view! { cx,
                            tr {
                                td(class="whatespace-nowrap px-3 py-4 text-sm") { (user.name) }
                                td(class="whatespace-nowrap px-3 text-sm") {
                                    (if Some(&user.name) == (*score_edit.get()).as_ref() { view! { cx,
                                        span(class="inline-block") {
                                            input(bind:value=new_score, class="mx-2 appearance-none block bg-gray-700 text-slate-400 border border-gray-600 rounded py-2 px-2 w-20 leading-tight focus:outline-none focus:ring-1 focus:ring-gray-500 focus:border-gray-500")
                                        }
                                        Button(icon="mdi-check".into(), onclick=oncheck)
                                    }} else { view! { cx,
                                        span(class="my-3,w-20") {
                                            (user.score)
                                        }
                                        Button(
                                            icon="mdi-pencil".into(),
                                            onclick=move |_| {
                                                score_edit.set(Some(user.name.clone()));
                                                new_score.set(user.score.to_string());
                                            }
                                        )
                                    }})
                                }
                                td(class="whatespace-nowrap px-3 py-4 text-sm") {
                                    Button(icon="mdi-delete".into(),onclick=ondelete(user.name.clone()))
                                }
                            }
                        }
                    },
                    key=|user| (user.name.clone(), user.score.clone()),
                )
                tr {
                    td(class="whatespace-nowrap px-3 text-sm") {
                        span(class="inline-block") {
                            input(bind:value=new_username, class="mx-2 appearance-none block bg-gray-700 text-slate-400 border border-gray-600 rounded py-2 px-2 w-50 leading-tight focus:outline-none focus:ring-1 focus:ring-gray-500 focus:border-gray-500")
                        }
                    }
                    td(class="whatespace-nowrap px-3 py-4 text-sm") {}
                    td(class="whatespace-nowrap py-4 px-3 text-sm") {
                        Button(icon="mdi-plus".into(),onclick=onadd)
                    }
                }
            }
        }
    }
}
